from typing import Union

from aipkgs_requests import status
from flask import make_response, jsonify
from enum import Enum

from aipkgs_core.responses.response import ErrorResponseInterface


class ErrorResponse(ErrorResponseInterface):

    @classmethod
    def forge_dict(cls, data: dict = None, error_code: Union[int, Enum] = None, status_code: int = None):
        response = {}
        if data:
            response.update(data)

        if error_code:
            response['error_code'] = error_code.value if isinstance(error_code, Enum) else error_code

        return make_response(jsonify(response), status_code or status.HTTP_400_BAD_REQUEST)

    @classmethod
    def forge(cls, title: str = None, message: str = None, message_code: str = None, error_code: Union[int, Enum] = None, data: dict = None, status_code: int = None):
        response = {'title': title or 'Warning',
                    'message': message or 'Something went wrong'}
        if message_code:
            response['message_code'] = message_code

        if data:
            response.update(data)

        if error_code:
            response['error_code'] = error_code.value if isinstance(error_code, Enum) else error_code

        return make_response(jsonify(response), status_code or status.HTTP_400_BAD_REQUEST)

    # @classmethod
    # def forge_localized(cls, title_key: str = None, message_key: str = None, error_code: Union[int, Enum] = None, status_code: int = None):
    #     return ErrorResponse.forge(title=gettext(title_key) if title_key else None, message=gettext(message_key) if message_key else None, error_code=error_code,
    #                                status_code=status_code)

    @classmethod
    def not_found_error(cls, message: str = None):
        return ErrorResponse.forge(message=message or "Not found", status_code=status.HTTP_404_NOT_FOUND)


class JWTModel:
    def __init__(self, jwt_data):
        self.jwt_data = jwt_data

        self.fresh = jwt_data.get('fresh')
        self.iat = jwt_data.get('iat')
        self.jti = jwt_data.get('jti')
        self.type = jwt_data.get('type')
        self.sub = jwt_data.get('sub')
        self.nbf = jwt_data.get('nbf')
        self.csrf = jwt_data.get('csrf')
        self.exp = jwt_data.get('exp')
        self.full_name = jwt_data.get('full_name')
        self.email = jwt_data.get('email')
        self.mobile = jwt_data.get('mobile')
        self.country_iso_code = jwt_data.get('country_iso_code')

    def is_fresh(self):
        return self.fresh

    def is_access(self):
        return self.type == 'access'

    def is_refresh(self):
        return self.type == 'refresh'

    def is_admin(self):
        return self.jwt_data.get('is_admin')



from functools import wraps

from aipkgs_requests import status
from flask_jwt_extended import get_jwt
from flask import request
import inspect

from aipkgs_core.responses.error import ErrorResponse


class FlaskDecorators:
    @staticmethod
    def is_admin():
        def decorator(func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                claims = get_jwt()
                if not claims["is_admin"]:
                    return ErrorResponse.forge(message="Admin privilege required.",
                                               status_code=status.HTTP_401_UNAUTHORIZED)
                return func(*args, **kwargs)

            return wrapper

        return decorator



import phonenumbers
from phonenumbers import PhoneNumber

from aipkgs_core.utils import utils


def is_number_format_valid(number: str) -> (bool, str):
    try:
        # if utils.contains_space(number):
        #     return False, number

        parsed_number: PhoneNumber = phonenumbers.parse(number, None)
        return True, parsed_number
    except:
        return False, number


def is_valid_number(number: str) -> (bool, PhoneNumber):
    is_valid, phone_number = is_number_format_valid(number)
    if not is_valid:
        return False, phone_number
    is_phone_valid = phonenumbers.is_valid_number(phone_number)
    return is_phone_valid, phone_number



from email_validator import validate_email, EmailNotValidError, ValidatedEmail

from aipkgs_core.utils import utils


def is_email_valid(email: str,
                   allow_smtputf8: bool = False,
                   check_deliverability: bool = False,
                   allow_empty_local: bool = False
                   ) -> (bool, str):
    try:
        if utils.contains_space(email):
            return False

        validated_email: ValidatedEmail = validate_email(email,
                                                         allow_smtputf8=allow_smtputf8,
                                                         check_deliverability=check_deliverability,
                                                         allow_empty_local=allow_empty_local)

        email = validated_email.ascii_email
        return True
    except EmailNotValidError as e:
        return False


def is_valid_email(email: str,
                   allow_smtputf8: bool = False,
                   check_deliverability: bool = False,
                   allow_empty_local: bool = False
                   ) -> bool:
    return is_email_valid(email,
                          allow_smtputf8=allow_smtputf8,
                          check_deliverability=check_deliverability,
                          allow_empty_local=allow_empty_local)



from flask import make_response, jsonify

from aipkgs_requests import status

from aipkgs_core.responses.response import VerboseResponseInterface


class VerboseResponse(VerboseResponseInterface):
    @classmethod
    def forge(cls, data: dict = None, status_code: int = None):
        response = {}
        if data:
            response.update(data)

        return make_response(jsonify(response), status_code or status.HTTP_200_OK)



from typing import TypeVar, Generic

T = TypeVar('T')


class Singleton(Generic[T]):
    """
    A non-thread-safe helper class to ease implementing singletons.
    This should be used as a decorator -- not a metaclass -- to the
    class that should be a singleton.

    The decorated class can define one `__init__` function that
    takes only the `self` argument. Also, the decorated class cannot be
    inherited from. Other than that, there are no restrictions that apply
    to the decorated class.

    To get the singleton instance, use the `instance` method. Trying
    to use `__call__` will result in a `TypeError` being raised.

    """

    def __init__(self, decorated):
        self._decorated: T = decorated

    def get_instance(self) -> T:
        """
                Returns the singleton instance. Upon its first call, it creates a
                new instance of the decorated class and calls its `__init__` method.
                On all subsequent calls, the already created instance is returned.

                """
        try:
            return self._instance
        except AttributeError:
            self._instance: T = self._decorated()
            return self._instance

    @property
    def shared(self) -> T:
        return self.get_instance()

    @property
    def instance(self) -> T:
        return self.get_instance()

    @property
    def singleton(self) -> T:
        return self.get_instance()

    def __call__(self):
        raise TypeError('Singletons must be accessed through `instance()`.')



from flask import make_response, jsonify
from aipkgs_requests import status

from aipkgs_core.responses.response import SuccessResponseInterface


class SuccessResponse(SuccessResponseInterface):
    @classmethod
    def forge_dict(cls, data: dict = None, status_code: int = None):
        response = {}
        if data:
            response.update(data)

        return make_response(jsonify(response), status_code or status.HTTP_200_OK)

    @classmethod
    def forge_raw(cls, status_code: int = None, **kwargs):
        response = {}
        response.update(kwargs)

        return make_response(jsonify(response), status_code or status.HTTP_200_OK)

    @classmethod
    def forge(cls, title: str = None, message: str = None, data: dict = None, status_code: int = None):
        response = {'title': title or 'success',
                    'message': message}
        if data:
            response.update(data)

        return make_response(jsonify(response), status_code or status.HTTP_200_OK)



import os
import pathlib
import shutil


class PathType:
    isPath = 0
    isDirectory = 1
    isFile = 2


def check_path_type(path):
    if os.path.isdir(path):
        return PathType.isDirectory
    elif os.path.isfile(path):
        return PathType.isFile
    else:
        return PathType.isPath


def directory_is_accessible(path):
    if (check_path_type(path) is PathType.isDirectory) and os.access(path, os.R_OK):
        return True
    else:
        return False


def create_dir(path: str):
    path = pathlib.Path(path)
    if directory_is_accessible(path):
        pass
    else:
        try:
            os.mkdir(path)
        except OSError:
            print("Creation of the directory %s failed" % path)
        else:
            print("Successfully created the directory %s " % path)

    return path


def create_dirs(path: str):
    path = pathlib.Path(path)
    if directory_is_accessible(path):
        pass
    else:
        try:
            os.makedirs(path)
        except OSError:
            print("Creation of the directory %s failed" % path)
        else:
            print("Successfully created the directory %s " % path)

    return path


def remove_dir(path: str):
    try:
        shutil.rmtree(path)
    except OSError as e:
        print(f'path: {path} could not be deleted')



import enum
import json


class FileAccessLevelEnum(enum.Enum):
    write = "w+"
    read = "r"


def create_file(name: str, content: str, access: FileAccessLevelEnum):
    file = open_file(name=name, access=access)
    file.write(content)


def open_file(name: str, access: FileAccessLevelEnum):
    file = open(name, access.value)
    return file


def close_file(file):
    file.close()


def open_json_file(path: str):
    with open(path) as f:
        return json.load(f)


def read_from_json_file(path: str, key: str, default: str = None):
    content_dict: dict = open_json_file(path=path)
    if key in content_dict:
        return content_dict[key]
    else:
        return default or key


